class Robot {
    constructor(height, width, posX, posY, color, scene) {
        this.height = height;
        this.width = width;
        this.x = posX;
        this.y = posY;
        this.color = color;
        this.range = WIDTH_MAP / 2;
        this.damage = DAMAGE;
        this.life = LIFE;
        this.shield = SHIELD;
        this.addScene(scene);
    }

    addScene(scene) {
        this.scene = scene;
        this.circle = this.scene.add.circle(this.x, this.y, this.width / 2, this.color);
    }

    setX(x) {
        this.x = x;
        this.circle.setX(this.x);
    }

    setY(y) {
        this.y = y;
        this.circle.setY(this.y);
    }

    setTarget(target) {
        this.target = target;
        if (this.target != null)
            this.updateTarget();
    }

    updateTarget() {
        this.setTargetPos(this.target.x, this.target.y);
    }

    setTargetPos(x, y) {
        let diffX = this.target.x - this.x;
        let diffY = this.target.y - this.y;
        let hypot = Math.hypot(diffX, diffY);
        this.velocityX = diffX / hypot * WIDTH_MAP / 1000 * SPEED;
        this.velocityY = diffY / hypot * WIDTH_MAP / 1000 * SPEED;
    }

    advanceToTarget() {
        if ((this.velocityX >= 0 && this.x >= this.target.x) || (this.velocityX <= 0 && this.x <= this.target.x))
            this.setX(this.target.x);
        else
            this.setX(this.x + this.velocityX);

        if ((this.velocityY >= 0 && this.y >= this.target.y) || (this.velocityY <= 0 && this.y <= this.target.y))
            this.setY(this.target.y);
        else
            this.setY(this.y + this.velocityY);

    }

    fleeFromTarget() {
        if (this.velocityX >= 0 && this.x - this.width / 2 <= 0)
            this.setX(this.width / 2);
        else if (this.velocityX <= 0 && this.x + this.width / 2 >= WIDTH_MAP)
            this.setX(WIDTH_MAP - this.width / 2);
        else
            this.setX(this.x - this.velocityX);

        if (this.velocityY >= 0 && this.y - this.width / 2 <= 0)
            this.setY(this.height / 2);
        else if (this.velocityY <= 0 && this.y + this.width / 2 >= WIDTH_MAP)
            this.setY(WIDTH_MAP - this.height / 2);
        else
            this.setY(this.y - this.velocityY);
    }


    attackTarget() {
        if (this.isTargetInRange()) {
            this.attack(this.target);
        }
        return false;
    }

    attack(target) {
        if (target.beAttack(this.damage)) {
            this.setTarget(null);
        }
    }

    beAttack(damage) {
        this.shield = this.shield - damage;
        if (this.shield < 0) {
            this.life += this.shield;
            this.shield = 0;
        }
        if (this.life <= 0) {
            this.life = 0;
        }
        return this.die();
    }

    isTargetInRange() {
        let hypot = Math.hypot(this.target.x - this.x, this.target.y - this.y);
        return hypot < this.range;

    }

    isAlive() {
        return this.life > 0;
    }

    die() {
        if (this.life === 0) {
            this.circle.destroy();
            return true;
        }
        return false;
    }
}
